@@ -57,6 +57,9 @@ from .forms import (
 from .models import Category, Consumption, Ingredient, MealType
 
 # Create your views here.
+
+meal_type_name_0 = _("Meal type 0")
+
 decimal_prec = getattr(settings, "DECIMAL_PREC", 2)
 
 storage_date_header = (
@@ -156,113 +159,40 @@ def split_price(total_price, quantity, prec=2):
     return [[total_price0, quantity0], [total_price1, quantity1]]
 
 
-def get_consumption_ingredients(request):
+@login_required
+def create_consumptions(request, ingredient_id=None):
     date_start = request.GET.get(
         "storage_date_start", None
     ) or request.COOKIES.get("storage_date_start", None)
-    date_end = request.GET.get("storage_date_end", None) or request.COOKIES.get(
-        "storage_date_end", None
-    )
-    ingredients = Ingredient.objects
-    queries = Q()
-    queries = (
-        Q(user=request.user) & Q(is_disabled=False) & Q(is_ignorable=False)
-    )
-    if date_start:
-        date_start = date_parser.parse(date_start).date()
-        queries &= Q(storage_date__gte=date_start)
-    if date_end:
-        date_end = date_parser.parse(date_end).date()
-        queries &= Q(storage_date__lte=date_end)
-
-    if not date_start and not date_end:
-        queries &= Q(quantity__gt=F("total_consumed"))
-        ingredients = ingredients.annotate(
+    if not date_start:
+        ingredients = Ingredient.objects.annotate(
             total_consumed=Coalesce(
                 Sum("consumptions__amount_used"), 0, output_field=IntegerField()
             )
-        )
-
-    ingredients = ingredients.filter(queries).order_by(
-        "name", "storage_date", "meal_type__name", "category__name"
-    )
-
-    return ingredients
-
-
-def get_date_range(ingredients):
-    date_start = ingredients.order_by("storage_date").first().storage_date
-    today = datetime.now().date()
-    date_end = (today.replace(day=1) + relativedelta(months=2)) - relativedelta(
-        days=1
-    )
-    date_range = list(pd.date_range(start=date_start, end=date_end))
-    date_range = [d.date() for d in date_range]
-
-    return date_range
-
-
-@login_required
-@require_POST
-def new_consumption(request, consumption_id=None):
-    form = None
-    if consumption_id:
-        consumption = Consumption.objects.filter(
-            Q(pk=consumption_id)
-            & Q(ingredient__user=request.user)
+        ).filter(
+            Q(quantity__gt=F("total_consumed"))
             & Q(is_disabled=False)
-        ).first()
-        if consumption:
-
-            posted_amount_used = request.POST.get("amount_used")
-            if (
-                posted_amount_used.replace(".", "").isnumeric()
-                and Decimal(posted_amount_used).is_zero()
-            ):
-                consumption.delete()
-                return HttpResponse("OK", status=201)
-
-            form = ConsumptionForm(request.POST, instance=consumption)
-        else:
-            return HttpResponse("Accepted", status=202)
+            & Q(is_ignorable=False)
+            & Q(user=request.user)
+        )
+        date_start = ingredients.order_by("storage_date").first().storage_date
 
     else:
-        ingredient = Ingredient.objects.filter(
-            Q(user=request.user) & Q(pk=request.POST.get("ingredient"))
-        ).first()
-        if not ingredient:
-            return HttpResponse("Accepted", status=202)
+        date_start = date_parser.parse(date_start).date()
 
-        consumption = Consumption()
-        consumption.ingredient = ingredient
-        form = ConsumptionForm(request.POST, instance=consumption)
-
-    if form.is_valid() and not form.instance.is_disabled:
-        consumption = form.save(commit=False)
-        consumption.save()
-        return HttpResponse("OK", status=201)
-
-    return HttpResponse("Accepted", status=202)
-
-
-@login_required
-def create_consumptions(request, ingredient_id=None):
-    ingredients = get_consumption_ingredients(request)
-    date_range = None
-    if not ingredients:
-        return render(
-            request,
-            "canteen/consumption/create.html",
-            {"ingredients": ingredients, "date_range": date_range},
-        )
-
-    date_range = get_date_range(ingredients)
     date_end = request.GET.get("storage_date_end", None) or request.COOKIES.get(
         "storage_date_end", None
     )
-    if date_end:
+    if not date_end:
+        today = datetime.now().date()
+        date_end = (
+            today.replace(day=1) + relativedelta(months=2)
+        ) - relativedelta(days=1)
+    else:
         date_end = date_parser.parse(date_end).date()
-        date_range = [d for d in date_range if d <= date_end]
+
+    date_range = list(pd.date_range(start=date_start, end=date_end))
+    date_range = [d.date() for d in date_range]
 
     if ingredient_id:
         ingredient = get_object_or_404(Ingredient, pk=ingredient_id)
@@ -270,6 +200,15 @@ def create_consumptions(request, ingredient_id=None):
         consumptions = ingredient.consumptions.filter(
             Q(is_disabled=False)
         ).all()
+
+        consumptions_len = len(consumptions)
+        for c_index in range(consumptions_len):
+            consumption = consumptions[c_index]
+            for c0_index in range(c_index + 1, consumptions_len):
+                consumption0 = consumptions[c0_index]
+                if consumption.date_of_using == consumption0.date_of_using:
+                    consumption.delete()
+
         consumption_dates = list(set([c.date_of_using for c in consumptions]))
 
         for per_day in date_range:
@@ -279,9 +218,6 @@ def create_consumptions(request, ingredient_id=None):
                     c for c in consumptions if c.date_of_using == per_day
                 ]
                 consumption = consumptions_per_day[0]
-                if len(consumptions_per_day) > 1:
-                    for c in consumptions_per_day[1:]:
-                        c.delete()
 
             else:
                 consumption = Consumption()
@@ -304,8 +240,47 @@ def create_consumptions(request, ingredient_id=None):
             "canteen/consumption/_create.html",
             {"form_list": form_list},
         )
+    by_week = (
+        request.GET.get("by_week", "") or request.COOKIES.get("by_week", "")
+    ) == "true"
+    rapidly = (
+        request.GET.get("rapidly", "") or request.COOKIES.get("rapidly", "")
+    ) == "true" or not by_week
+
+    queries = (
+        Q(storage_date__lte=date_end)
+        & Q(user=request.user)
+        & Q(is_disabled=False)
+        & Q(is_ignorable=False)
+    )
+    ingredients = Ingredient.objects
+
+    ingredients = (
+        ingredients.filter(queries).order_by(
+            "name", "storage_date", "meal_type__name", "category__name"
+        )
+        if rapidly
+        else ingredients.filter(queries).order_by(
+            "storage_date", "meal_type__name", "name", "category__name"
+        )
+    )
 
     ingredients = ingredients.all()
+
+    if not ingredients:
+        return render(
+            request,
+            "canteen/consumption/create.html",
+            {"ingredients": ingredients, "date_range": date_range},
+        )
+
+    ingredients = [
+        i
+        for i in ingredients
+        if i.get_remaining_quantity(date_end) > 0
+        or i.storage_date >= date_start
+    ]
+
     ingredients_pinned = []
     ingredients_unpinned = []
     categories_top = Category.objects.filter(
@@ -322,6 +297,9 @@ def create_consumptions(request, ingredient_id=None):
 
     ingredients = ingredients_pinned + ingredients_unpinned
 
+    for ingredient in ingredients:
+        ingredient.quantity_used = ingredient.get_consuming_quantity(date_start)
+
     date_range_cp = date_range
     date_range_cp = [d.strftime("%Y-%m-%d") for d in date_range]
     meal_types = list(set([i.meal_type.name for i in ingredients]))
@@ -337,10 +315,66 @@ def create_consumptions(request, ingredient_id=None):
             "meal_types": meal_types,
             "ingredient_ids": ingredient_ids,
             "months": months,
+            "storage_date_start": date_start,
+            "meal_type_name_0": meal_type_name_0,
         },
     )
 
 
+@login_required
+@require_POST
+def new_consumption(request, consumption_id=None):
+    form = None
+
+    posted_date_of_using = date_parser.parse(request.POST.get("date_of_using"))
+    posted_amount_used = request.POST.get("amount_used")
+
+    if consumption_id:
+        consumption = Consumption.objects.filter(
+            Q(pk=consumption_id)
+            & Q(ingredient__user=request.user)
+            & Q(date_of_using=posted_date_of_using)
+            & Q(is_disabled=False)
+        ).first()
+        if consumption:
+            if Decimal(posted_amount_used).is_zero():
+                consumption.delete()
+                return HttpResponse("OK", status=201)
+
+            form = ConsumptionForm(request.POST, instance=consumption)
+        else:
+            return HttpResponse("Accepted", status=202)
+
+    else:
+        ingredient_id = request.POST.get("ingredient")
+        ingredient = Ingredient.objects.filter(
+            Q(user=request.user) & Q(pk=ingredient_id)
+        ).first()
+        if not ingredient:
+            return HttpResponse("Accepted", status=202)
+
+        consumption = Consumption.objects.filter(
+            Q(date_of_using=posted_date_of_using)
+            & Q(ingredient__id=ingredient_id)
+        ).first()
+
+        if not consumption:
+            consumption = Consumption()
+        elif Decimal(posted_amount_used).is_zero():
+            consumption.delete()
+            return HttpResponse("OK", status=201)
+
+        consumption.ingredient = ingredient
+        form = ConsumptionForm(request.POST, instance=consumption)
+
+    if form.is_valid() and not form.instance.is_disabled:
+        consumption = form.save(commit=False)
+        consumption.save()
+        return HttpResponse("OK", status=201)
+
+    return HttpResponse("Accepted", status=202)
+
+
 @login_required()
 def delete_ingredient(request, ingredient_id):
     ingredient = get_object_or_404(Ingredient, pk=ingredient_id)
@@ -493,6 +527,11 @@ def create_ingredients(request):
             for index, row in df.iterrows():
                 category_name = row[category_header[0]]
                 meal_type_name = row[meal_type_header[0]]
+                meal_type_name = (
+                    meal_type_name_0
+                    if meal_type_name is np.nan
+                    else meal_type_name
+                )
 
                 category = Category.objects.filter(
                     Q(name=category_name) & Q(user=request.user)
@@ -780,7 +819,6 @@ class CategoryListView(LoginRequiredMixin, ListView):
     model = Category
     template_name = "canteen/category/list.html"
     context_object_name = "categories"
-    ordering = ["-created_at"]
 
     paginate_by = 10
     paginate_orphans = 2
